package com.yeejoin.amos.fas.business.action.mq;

import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.ListenableFutureCallback;

/**
 * @author keyong
 * @title: KafkaProducer
 * <pre>
 * @description:
 * </pre>
 * @date 2022/10/14 10:59
 */
@Configuration
@EnableKafka
public class KafkaProducer implements IKafkaProducer {

    private final Logger log = LoggerFactory.getLogger(KafkaProducer.class);

    @Value("${kafka.producer.sysIsUsed}")
    private String sysIsUsed;

    @Autowired
    private KafkaTemplate<String, Object> kafkaTemplate;

    private static class KafkaFutureCallback implements ListenableFutureCallback<SendResult> {

        private final Logger log = LoggerFactory.getLogger(KafkaProducer.class);

        @Override
        public void onFailure(Throwable ex) {
            log.error("send kafka failed: {}", ex);
        }

        @Override
        public void onSuccess(SendResult result) {
            log.info("send kafka success: {}", result.getProducerRecord());
        }
    }

    @Override
    public void sendMessage(String topic, String key, Object message) {
        if("on".equalsIgnoreCase(sysIsUsed)){
            log.info("Kafka Topic===="+topic);
            log.info("Kafka msg===="+ JSON.toJSONString(message));
            kafkaTemplate.send(topic, key, message).addCallback(new KafkaFutureCallback());
        }else{
            log.info("Kafka SendResult====kafka.producer.sysIsUsed is not open !!!");
        }
    }
}
